/* this includes the two files which make up my simple FITS to HDF
   converter.  The FITS reader program starst at the comment line 
   "File number 2".  Note that this program will take the FITS
   file that is produced by IRAF and convert it to an HDF file
   which can be read in by COLLAGE */

#include <stdio.h>
#include "/local/NCSA/hdf/include/df.h"
#include <malloc.h>

float **rfits(FILE *, int *, int *);

main(int narg, char *args[])
{
  int i,j;
  char infile[20],outfile[20];
  float *xscale,*yscale;
  FILE *in;
  float **pixels;
  int nx,ny;
  float *pixs,*xarray,*yarray;
  int rank;
  int shape[2];


  /* open up the files for reading and writing */

  if(narg<2){
    printf("Where does it come from: ");
    scanf("%s",infile);
  }
  else strcpy(infile,args[1]);
  while((in=fopen(infile,"r"))==NULL){
    fprintf(stderr,"Could not find file %s\n",infile);
    exit(30);
  }
  if(narg<3){
    printf("And where is it going: ");
    scanf("%s",outfile);
  }
  else strcpy(outfile,args[2]);
  
  /* read in the fits data */

  pixels = rfits(in,&nx,&ny);

  /* set up for the hdf stuff...its 2d (rank) and has a shape of nx x ny */
  rank = 2;
  shape[0]=nx;
  shape[1]=ny;

  /* because malloc allocates ATLEAST so many bytes....we need to copy
     this into a oned array so the pixels are in contiguous memory...sigh
     So allocate a 1d array with the right size and copy in the 2d array
     accordingly */

  pixs = (float *) malloc (nx*ny*sizeof(float));
  xarray = (float *) malloc (nx*sizeof(float));
  yarray = (float *) malloc (ny*sizeof(float));

  for(i=0;i<nx;i++) xarray[i] = i + 1.0;
  for(i=0;i<ny;i++) yarray[i] = i + 1.0;

  for(i=0;i<nx;i++){
    for(j=0;j<ny;j++){
      pixs[i*ny+j]=pixels[i][j];
    }
  }
  
  /* tell the hdf file what your giving it in the comming array */

  if(DFSDsetdims(rank,shape)){
    fprintf(stderr,"DDFSDsetdims error\n");
  }

  if(DFSDsetdimscale(1,nx,xarray)){
    fprintf(stderr,"DDFSDsetdimscale x error\n");
  }

  if(DFSDsetdimscale(2,ny,yarray)){
    fprintf(stderr,"DDFSDsetdimscale y error\n");
  }


  

  
  /* next give it the array */

  if(DFSDputdata(outfile,rank,shape,pixs)){
    fprintf(stderr,"DDSDputdata error\n");
  }
  fclose(in);
}

/* File number 2 */
#include <stdio.h>
#include <malloc.h>
#include <string.h>


/* program reads in a 2d fits image, allocates a 2d array for it
   and then returns the image and the size of the array */

float **rfits(FILE *file,int *nx,int *ny)
     
{
  int count=0,i,j=0,k=0;
  int naxis1,naxis2;
  char param[9],entry[2880];
  char *pt;
  char *rest;
  float fy;
  short sy;
  long int ly;
  double wpc,b0,bscale;
  int bitpix;
  int header = 1;
  char title[80];
  float **y;


  /*read in the FITS header in 2880 blocks looking for 
   the important key words*/

  while(header){ fread(entry,2880,1,file);
    count = 0;
    for(pt=&entry[0];count <= 2880; pt+=80){
      sscanf(pt,"%8s",param);
      rest = pt + 10; 
      if(strcmp(param,"BZERO")==0) { /* zero offset for integer data */
	sscanf(rest," %lf ",&b0);
      }
      if(strcmp(param,"BSCALE")==0) { /*scale for integer data */
	sscanf(rest," %lf ",&bscale);
      }
      if(strcmp(param,"BITPIX")==0) { /* bits per pixel (negative for
					 ieee floats) */
	sscanf(rest," %d ",&bitpix);
      }
      if(strcmp(param,"NAXIS1")==0) { /* size of axis 1 */
	sscanf(rest," %d ",&naxis1);
      }
      if(strcmp(param,"NAXIS2")==0) { /* size of axis 2 */
	sscanf(rest," %d ",&naxis2);
      }
      
      if(strcmp(param,"END") == 0) header = 0;
      count += 80;
    }
  }

  /* next make a 2d array of size naxis2 by naxis1 */

  y = (float **) calloc(naxis2,sizeof(float *));
  if(!y) 
    fprintf(stderr,"allocation failure 1 in matrix()");
  for(i=0;i<=naxis2;i++) {
    y[i]= (float *) calloc(naxis1,sizeof(float));
    if (!y[i]) 
      fprintf(stderr,"allocation failure 2 in matrix()");
  }

  /* then read in the nubers according to bitpix (16 = short 32 = long
     and - = float */

  if(bitpix > 0){
    if(bitpix==16){
      for(k=0;k<naxis2;k++){
	for(j=0;j<naxis1;j++){
	  fread(&sy,sizeof(short),1,file);
	  y[k][j]=sy*bscale+b0;
	}
      }
    }
    else {
      for(k=0;k<naxis2;k++){
	for(j=0;j<naxis1;j++){
	  fread(&ly,sizeof(long int),1,file);
	  y[k][j]=ly*bscale+b0;
	}
      }
    }
  }
  else {
    for(k=0;k<naxis2;k++){
      for(j=0;j<naxis1;j++){
	fread(&fy,sizeof(float),1,file);
	y[k][j]=fy;
      }
    }
  }

  /* return the size of the array in x and y and the array itself */
  
  *nx = naxis2;
  *ny = naxis1;
  return(y);
}
  
      
